package relayServer;

import java.io.*;
import java.util.*;
import java.net.*;
import java.util.logging.*;

import data.*;

/**
 * Deze klasse zorgt ervoor dat e-mails die niet 
 * op de lokale server thuis horen, doorgestuurd worden naar
 * een andere server.
 * 
 * Normaal wordt dit via een MX DNS look-up gedaan. 
 * Echter doordat relaying vaak dichtstaat, wordt er gebruik gemaakt van een hostfile.
 * Deze staat onder settings/relayservers, hierin staat het domein en ip adres van 
 * de bekende servers.
 *
 *@version 0.65
 *TODO Mogelijkheid om settingsvariabelen te kunnen veranderen (evt. via constructors).
 *TODO [OPT] Toevoegen van een default relaying server.
 *TODO [OPT] Invoegen van DNS MX lookup.
 */

public class RelayServer extends Thread {	
	//Settings variabelen.
	private int sleepTime = 10000;	//5minuten = 300000
	private String relayPath = "data/relay/";
	private String sentPath = "data/sent/";
	private String settingsPath = "data/settings/";
	private String localHostname = "onzeserver.nl";
	private String defRelayServer = "10.0.0.3";
	
	//Logger
	private Logger log;
	
	//Globale variabelen
	private boolean relaying = true;
	private RLYState rlyState;
	private BufferedReader input;
	private PrintWriter output;
	
	public RelayServer(){
		log = Logger.getLogger("RelayServer");
	}
	
	/**
	 * De run start de server, deze wordt aangeroepen met object.start().
	 * Anders wordt er geen nieuwe thread gestart.
	 */
	public void run(){
		log.info("RelayServer succesvol gestart.");
		while (relaying){
			checkAndSent();
			//Een sleep, zodat de thread niet constant aan het draaien is.
			try{
				sleep(sleepTime);
			}
			catch(Exception ex){
				log.warning("Fout opgetreden, service vastgelopen.");
				ex.printStackTrace();
			}
		}
	};
	
	
	/**
	 * Controlleert of er berichten klaar staan en verstuurd deze.
	 * Berichten worden opgepikt in (relayPath).
	 * Er wordt gekeken of de domain van deze berichten bekend is, zo
	 * ja wordt er geprobeerd deze berichten door te sturen.
	 */
	public void checkAndSent(){
		File dir = new File(relayPath);
		File chkFile;
		File[] files;
		Message msg;
		String hostIP;
		Socket host;
		boolean msgExists;
		int cntForFileName;
		String fileName;
		String file = null;
		
		//Controleren of er berichten staan
		if (dir.exists()){
			files = dir.listFiles();
			
			log.info("Controleren op berichten voor relaying");
			
			if (files.length > 0){
				log.info("Er zijn " + files.length + "bericht(en) voor relaying");
				//Alle gevonden berichten doorlopen
				for(int i=0; i < files.length; i++){
					//Berich openen
					msg = Message.getMessage(files[i]);
					log.info("Bericht header geopend. zoeken naar domein.");
					
					//Server zoeken
					hostIP = getIPAddress(msg.getDomain());
					if (hostIP != null){
						//Bericht verzenden
						log.info("Bericht doorsturen middels server: " + hostIP);
						try{
							host = new Socket(hostIP, 25);					
							input = new BufferedReader(new InputStreamReader(host.getInputStream()));
							output = new PrintWriter(new OutputStreamWriter(host.getOutputStream()));
							
							rlyState = new WAITState();
							
							//Commando's uitvoeren om het bericht te versturen
							while (rlyState.getState()){
								rlyState.doNextCmd(this, msg);
							}
							if (rlyState.getSent()){
								//Bericht is succescol verstuurd
								//Bericht kopiëren naar de data/sent map							  						    							    
							    File sentDir = new File(sentPath);
							    log.info("Bericht verzonden, kopie wordt opgeslagen.");
							    //Zorgen dat de directory bestaat
							    if (!sentDir.exists()){
							    	sentDir.mkdirs();
							    }
							    
							    //Zorgen voor een unieke naam.
							    msgExists = true;
							    fileName = "sent_msg_";
							    cntForFileName = 0;
							    while (msgExists){
							    	cntForFileName++;
							    	file = fileName + cntForFileName + ".txt";
							    	chkFile = new File(sentPath+file);
							    	if (!chkFile.exists()){
							    		msgExists = false;
							    	}
							    }
							    
							    //Bestand verplaatsen.
							    files[i].renameTo(new File(sentDir, file));
							    log.info("Bericht opgeslagen onder: " + sentDir + file);
							}
							
						}
						catch(Exception ex){
							log.warning("Fout opgetreden bij connecteren naar server.");
							ex.printStackTrace();
						}
					}
	
				}
				//Geen bestanden gevonden
			}
			//Alles herhalen
		}
		else{
			//Directory niet gevonden.
			//Logging wegschrijven
			return;
		}		
		
	}
	
	/**
	 * Geeft de locale hostname terug.
	 * @return String hostname
	 */
	public String getLocalHostname(){
		return localHostname;
	}
	
	/**
	 * Zorgt ervoor dat de status veranderd kan worden.
	 * @param state
	 */
	public void setState(RLYState state){
		rlyState = state;
	}
	
	/**
	 * Schrijf naar de geconnecteerde server.
	 * Wordt gebruikt om commando's door te sturen (HELO, MAIL, etc.)
	 * @param command
	 */
	public void setOutput(String command){
		if (output != null){
			try{
				output.println(command);
				output.flush();
			}
			catch(Exception ex){
				ex.printStackTrace();
			}
		}
	}
	
	/**
	 * Leest de input van geconnecteerde server.
	 * @return input
	 */
	public String getInput(){
		if (input != null){
			try {
				return input.readLine();
			}
			catch(Exception ex){
				ex.printStackTrace();
				return null;
			}
		}
		else{
			return null;
		}
	}
	
	/**
	 * Zoekt het IP-adres op van de bijbehorende mailserver
	 * Dit wordt opgezocht in de hostfile relayservers
	 * @param domain
	 * @return hostIP
	 */
	private String getIPAddress(String domain){
		File relay = new File(settingsPath+"relayservers");
		BufferedReader relayServers;
		String line, hostname, hostip;
		StringTokenizer st;
		try{
			relayServers = new BufferedReader(new FileReader(relay));
			while ((line = relayServers.readLine()) != null){
				st = new StringTokenizer(line, "\t");
				hostname = st.nextToken();
				hostip = st.nextToken();
				if (hostname.equalsIgnoreCase(domain)){
					return hostip;
				}
			}
			return defRelayServer;
		}
		catch(Exception ex){
			ex.printStackTrace();
			return null;
		}
	}
}
